hdr;

load pts.dat;    p=pts; load tri.dat;  t=tri;

E =size(t,1);
nv=size(t,2);
nL=E*nv;
nb=size(p,1);  %%% THIS MAY BE AN OVERESTIMATE
tmp=zeros(nb,1);
tmp(reshape(t,3*E,1))=1;
nb=sum(tmp);

[AL,BL,Q]=local_fem_mats(p,t);

%% Set Dirichlet BCs

[maskL,gbdry] = get_maskl(t);

%%   BC = 0 %%  pure Neumann
%%        1 %%  dO_D = interior disk
%%        2 %%  dO_D = exterior box 
%%        3 %%  dO_D = interior disk + exterior box 


  rbdry = sqrt(p(gbdry,1).^2+p(gbdry,2).^2);
  ibbox = find(rbdry > 0.41);
  idisk = find(rbdry < 0.41);

  dbdry = [];
  if BC==1;
     dbdry = gbdry(idisk);
  elseif BC==2;
     dbdry = gbdry(ibbox);
  elseif BC==3;
     dbdry = gbdry;
  end;

  gbdry = dbdry;

%%% Set up restriction/permutation matrix to make boundary nodes last

ir=[1:nb]';

n = nb-size(gbdry,1);
ir(gbdry) = ir(gbdry) + 2*nb;
[is,ind]=sort(ir); ind=ind(1:n);
P=speye(nb); P=P(:,ind); R=P';

%%%% Plot a function that is 1 in the interior, 0 on the boundary.

f=ones(n,1);   %% 1 in Interior
fb=R'*f;       %% Extend by 0 to the boundary.

hold off; tri=trimesh(t,p(:,1),p(:,2),fb);          %% Surface Plot
hold on;  plot(p(gbdry,1),p(gbdry,2),'r.',ms,9);    %% Plot boundary nodes in red.
axis equal; 
pause(1)


%%% Solve a PDE: -nabla^2 u = f(x,y), with f = y/x

rad = sqrt(p(:,1).^2+p(:,2).^2);
rad = rad(reshape(t,3*E,1));   %% Here, a continuous function of x and y
fL  = sin(5*pi*rad);                                   %% Here, a continuous function of x and y
fe=reshape(fL,E,3);
fe=fe';
fL=reshape(fe,3*E,1);
fL=.2+fL*5;

Ab = Q'*AL*Q; A = R*Ab*R';
Bb = Q'*BL*Q; B = R*Bb*R';


u  = A \ (R*(Q'*(BL*fL))); ue=u;
ub = R'*u;
hold off; tri=trimesh(t,p(:,1),p(:,2),ub);          %% Surface Plot
hold on;  plot(p(gbdry,1),p(gbdry,2),'r.',ms,9);    %% Plot boundary nodes in red.
axis equal; 

pause(1)

%%%% Solve using Jacobi Iteration

D=diag(A); Di=1./D;

ue2 = ue'*B*ue;
u = 0*u;
b = R*(Q'*(BL*fL));
for k=1:1000;

 u = u + Di.*(b-A*u);

 er = ue-u; erel(k) = sqrt( (er'*B*er)/ue2 );

 if k<20 || mod(k,50)==0; 
  eb = R'*(u-1.0*ue); err=max(abs(eb)); eb=abs(eb)/err; 
  hold off; tri=trimesh(t,p(:,1),p(:,2),eb);      %% Surface Plot
  hold on;  plot(p(gbdry,1),p(gbdry,2),'r.',ms,9);%% Plot boundary nodes in red.
  axis equal; 
  title(['Error/Soln, Iter: ' int2str(k) ', Error: ',num2str(err)],fs,18);
  xlabel('X',fs,18); ylabel('Y',fs,18); pause(.1);
 end;

end;

figure;
kk=1:k; kk=kk';
semilogy(kk,erel,'r.',ms,9);

DiA=diag(Di)*full(A);
lam=eig(DiA);
bc=mod(BC+4-1,4)+1 
lmin(bc)=min(lam)
lmax(bc)=max(lam)

